## 问题汇总及解决方案

### 1. user表性能分析与数据处理
1. user表的主键为user_id,但是实际业务中标示用户的唯一字段为user_md（md5序列），因此需要在user_md上创建一个普通索引来加快查询速度。  
在导入数据的时候，发现有一部分数据nickname重复，在用nickname和password做登录时，因为nickname不唯一，因此会造成登录错误，所以需要对数据进行清洗，  
清洗方式如下：  

```sql
UPDATE `user` set user_nickname=CONCAT(user_nickname,user_id) WHERE user_nickname in (SELECT u3.user_nickname FROM (SELECT u2.user_nickname from `user` u2 GROUP BY u2.user_nickname HAVING COUNT(*)>1) AS u3)
```

对nickname的唯一性维护在程序中实现，不在数据层面做约束，从而降低数据库创建索引的存储和时间消耗。

### 登录问题

用户登录后会以 key: token value:userMd的信息向redis中保存；登录后前端在request的header中带着token,  
后端通过自定义注解@LoginRequrie来进行登录验证（根据token获取userMd然后查询用户是否存在）

### 部署问题
服务器宽带太小，直接在服务器上使用docker build构建镜像速度太慢，所以搞了个github的公开仓库，来存储和分发镜像。    
并使用开发机构建镜像，push到github上来加速构建和部署速度。  
同时修改了服务器的docker代理，来加速镜像的拉取。  

### 性能问题
1、发现在更新用户信息时接口响应较慢，排查发现user表没有创建索引(肯定是智障了....)，因此对user表user_md和user_nickname创建唯一索引  


### 其他问题
发现服务器内存使用过高，导致docker服务oom，查看当前进程的内存使用情况发现node服务占用的资源太多，先kill掉。  
然后将问题定位到前端服务部署方式，在原来的方式中启动了一个node服务，然后使用 `npm run dev`直接启动了前端项目，这样导致占用资源太多。  
解决方法，直接使用`npm run build`将静态资源打出来，起个http的服务，暴露资源就可以  
同时启用swap，设置大小为2G  （ps: 因为服务器太弱鸡踩了好多坑，好像有台大的服务器，可惜我没有钱...太惨了...）  
